////////////////////////////////////////////////////////////////////////////////
//	Adam Sarzynski
//	adam@wobit.com.pl
//  SDD287 DUAL DC Motor Driver
//  ATmega128
//  30-07-07 12:21
////////////////////////////////////////////////////////////////////////////////

#include "Main.h"

////////////////////////////////////////////////////////////////////////////////
// Polaczenie modulu VHN3SP30 DAUL DC MOTOR DRIVER z mikrokontrolerem
// -----------------------------------------------
// |                                             |                                             
// |                                             |
// |                                             |
// |    SDD287  DUAL DC MOTOR DRIVER BOARD       |
// |                                             |
// |                                             |
// | P1 EN1 B1 A1 I1 GND     GND I2 P2 EN2 B2 A2 |  - wyjscia/wejscia modulu 
// -----------------------------------------------
//    |  |   |  |                    |  |   |  |                                                              
//    E3 C0  A3 A4                   E4 C1  A5 A6   - Piny mikrokontrolera (ATMEGA 128)
//  
//
// EN1 i EN2 podcigniete do +5V rezystorem ok 4,7k
// P1,P2 - PWM PIN
// EN1, EN2 - DIAG PIN
// A1, B1, A2, B2 - IN PIN
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
//Inicjalizacja obslugi mostkow 
////////////////////////////////////////////////////////////////////////////////
void drive_init(void)				
{
    //Mostek A
	DRIVE_PWM_DDR|=(1<<DRIVE_PWMA_PIN);        	//pin PWM jako wyjscia
	DRIVE_DDR|=(1<<IN1_A)|(1<<IN1_B);
	DRIVE_DIAG_DDR&=~(1<<DRIVE_DIAG_A_PIN);
	DRIVE_DIAG_PORT|=(1<<DRIVE_DIAG_A_PIN);
		
	//Mostek B
	DRIVE_PWM_DDR|=(1<<DRIVE_PWMB_PIN);	        //pin PWM jako wyjscia
	DRIVE_DDR|=(1<IN2_A)|(1<<IN2_B);
	DRIVE_DIAG_DDR&=~(1<<DRIVE_DIAG_B_PIN);
	DRIVE_DIAG_PORT|=(1<<DRIVE_DIAG_B_PIN);
	
	//Inicjalizacja PWM mostkow A i B
	TCCR3A=0;
	//TCCR3A|=(1<<COM3A1)|(1<<COM3B1)|(1<<WGM31)|(1<<WGM30);  // PWM, 10 - bit							
	TCCR3A|=(1<<COM3A1)|(1<<COM3B1)|(1<<WGM30);  // PWM, 8 - bit 
	TCCR3B=0;
	TCCR3B|=(1<<CS31)|(1<<CS30);		//przez 64
	//TCCR3B|=(1<<CS31);
	
}

////////////////////////////////////////////////////////////////////////////////
//Procedura jazdy
// left_speed - predkosc dla silnika mostka A ( wartosc z zakresu -255 do 255 )
// left_speed - predkosc dla silnika mostka B ( wartosc z zakresu -255 do 255 )
////////////////////////////////////////////////////////////////////////////////
void drive(signed int left_speed,signed int right_speed)
{
static signed int last_left_speed, last_right_speed;

//if((left_speed==last_left_speed)&&(right_speed==last_right_speed)) return;
  
// Mostek A

if(!(DRIVE_DIAG_PIN&(1<<DRIVE_DIAG_A_PIN)))	//jesli wykryto zadzialanie zabezpieczenia w napedzie
{
	DRIVE_PORT&=~(1<<IN1_A);           //zeruj piny sterujace
	DRIVE_PORT&=~(1<<IN1_B);
	OCR3A=0;								//zeruj napiecie zasilajace
	//UART_print("Przeciazenie mostka 1 (L)"); 
    Delay(500);
    DRIVE_PORT|=(1<<IN1_A);            	
	DRIVE_PORT|=(1<<IN1_B);
}
  
      
if(left_speed*last_left_speed<0)			//jesli zmienil sie znak zadanej predkosci
{
 	OCR3A=0;								//procedura soft-reverse :)
 	Delay(200);
 	DRIVE_PORT&=~(1<<IN1_A);
    DRIVE_PORT&=~(1<<IN1_B);
  	Delay(200);
}


if(left_speed>0)       						//ustawienie pinow sterujacych
{
	DRIVE_PORT|=(1<<IN1_A);
	DRIVE_PORT&=~(1<<IN1_B);
}
else if(left_speed<0)
{
	DRIVE_PORT&=~(1<<IN1_A);
	DRIVE_PORT|=(1<<IN1_B);
}

// Mostek B

if(!(DRIVE_DIAG_PIN&(1<<DRIVE_DIAG_B_PIN)))	//jesli wykryto zadzialanie zabezpieczenia w napedzie
{
	DRIVE_PORT&=~(1<<IN2_A);          //zeruj piny sterujace
	DRIVE_PORT&=~(1<<IN2_B);
	OCR3B = 0;
	//UART_print("Przeciazenie mostka 2 (r)"); 
	Delay(500);								//zeruj napiecie zasilajace
    DRIVE_PORT|=(1<<IN2_A);            	
	DRIVE_PORT|=(1<<IN2_B);
   

}
  
     
if(right_speed*last_right_speed<0)			//jesli zmienil sie znak zadanej predkosci
{
 	OCR3B=0;								//procedura soft-reverse :)
 	Delay(200);
 	DRIVE_PORT&=~(1<<IN2_A);
    DRIVE_PORT&=~(1<<IN2_B);
    Delay(200);

}

if(right_speed>0)       					//ustawienie pinow sterujacych
{
	DRIVE_PORT|=(1<<IN2_A);
	DRIVE_PORT&=~(1<<IN2_B);
}
else if(right_speed<0)
{
	DRIVE_PORT&=~(1<<IN2_A);
	DRIVE_PORT|=(1<<IN2_B);
}

OCR3A=(unsigned char)(abs(left_speed));      
OCR3B=(unsigned char)(abs(right_speed));      

last_left_speed=left_speed;					//przepisanie starej predkosci
last_right_speed=right_speed;

flaga_L = 1;
flaga_R = 1;

}

////////////////////////////////////////////////////////////////////////////////
// Procedura hamowania silnikow 
//  1 - hamowanie aktywne, 0 - hamowanie nieaktywne 
////////////////////////////////////////////////////////////////////////////////
void motor_break (unsigned char left_motor ,unsigned char right_motor)
{
     
   if(left_motor)
   {
  	    OCR3A=0;         
      	DRIVE_PORT&=~(1<<IN1_A);
    	DRIVE_PORT|=(1<<IN1_B);
   }  
    
    if(right_motor)
   {
  	    OCR3B=0;
      	DRIVE_PORT&=~(1<<IN2_A);
    	DRIVE_PORT|=(1<<IN2_B);
   }  
   
}


////////////////////////////////////////////////////////////////////////////////
//Opoznienie w ms (najwieksza wartosc okolo 16ms)
////////////////////////////////////////////////////////////////////////////////
inline void delay(unsigned int miliseconds)
{
	_delay_loop_2(miliseconds*delay_mul);
}
////////////////////////////////////////////////////////////////////////////////
//Opoznienie w ms (najwieksza wartosc okolo minuty)
////////////////////////////////////////////////////////////////////////////////
void Delay(unsigned long  miliseconds)
{
	for(unsigned long  i=miliseconds;i;i--)
		_delay_loop_2(delay_mul);
}



////////////////////////////////////////////////////////////////////////////////
//MAIN
////////////////////////////////////////////////////////////////////////////////
int main(void)
{
 unsigned char buf[32];
 UART_init();				//inicjalizacja RS
 drive_init();

 sei();						//uruchomienie globalne przerwan

DDRD|=(1<<PD1);

UART_print("SDD287  TEST"); 
flaga_L = 1;
flaga_L = 1;
///////////////////////////////////////////////////////////////////////////////
//Glowna petla programu
////////////////////////////////////////////////////////////////////////////////

for(;;)
{
		//PORTD|=(1<<PD1);

		//PORTD&=~(1<<PD1);
		
        if(UART_data_in_rx_buffer())
        {
    		
    		UART_gets(buf,16);		//odUART_gets polecenia z rsa
    		interpreter(buf);		//interpretuj polecenia
        }
		//PORTD&=~(1<<PD1);

		if((!(DRIVE_DIAG_PIN&(1<<DRIVE_DIAG_A_PIN)))&&flaga_L)
		{

			UART_print("Przeciazenie mostka 1 (L)");
			flaga_L = 0;
		}

      /*
		if((!(DRIVE_DIAG_PIN&(1<<DRIVE_DIAG_B_PIN)))&&flaga_R)
		{

			UART_print("Przeciazenie mostka 2 (R)");
			flaga_R = 0;
		}
		*/
		
	
}
////////////////////////////////////////////////////////////////////////////////
return 0;
}



